from db_paribhasha import db_constants
from db_paribhasha.base_models.sql_base_model import Base

from sqlalchemy import Column, Integer, String, Boolean, ForeignKey, JSON, DateTime
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func,expression


class WorkflowRequest(Base):
    """
    Defines a workflow request Model

    Defines a user instantiation/run of a workflow. A record in this table is
    made when a user starts/requests a workflow. This table also consists of
    required information corresponding to that request such as the workflow_id.
    """
    __tablename__ = db_constants.WORKFLOW_REQUEST_TABLE_NAME
    id = Column(Integer, primary_key=True, nullable=False,
                   autoincrement=True)
    name = Column(String)
    current_step = Column(Integer, nullable=False, default=0,
                             server_default="0")
    created_time = Column(DateTime, server_default=func.now())
    modified_time = Column(DateTime, server_default=func.now(), onupdate=func.now())

    state = Column(String(20), default='init',
                      server_default="init")
    owner = Column(Integer, ForeignKey(
        db_constants.USER_TABLE_NAME + ".id", ondelete="CASCADE"), nullable=False)
        
    input_file_names = Column(JSON)
    created_time = Column(DateTime, server_default=func.now(),
                             nullable=False)
    modified_time = Column(DateTime, server_default=func.now(),
                              onupdate=func.now(), nullable=False)
    active = Column(Boolean, default='t', nullable=False, server_default=expression.true())
    iterations = relationship('Iteration')

    def as_dict(self):
        """
        Returns the model as dict.
        """
        return_object = {
            'id': self.id,
            'name':self.name,
            'active': self.active,
            'current_step': self.current_step,
            'state': self.state,
            'owner': self.owner,
            'input_file_names' : self.input_file_names,
            'created_time': self.created_time,
            'modified_time': self.modified_time
        }
        return return_object
